/*
 * Project name:
     DHT22 click (Demonstration of mikroBUS DHT22 click board)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
     20140225:
       - initial release (FJ);
 * Description:
     This code demonstrates how to use DHT22 click with EasyPIC Fusion v7 board.
     DHT22 sensor measures temperature and relative humidity.
 * Test configuration:
     MCU:             P32MX795F512L
                      http://ww1.microchip.com/downloads/en/DeviceDoc/61156F.pdf
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.000MHz
                      http://www.mikroe.com/store/components/displays/
     SW:              mikroC PRO for PIC32
                      http://www.mikroe.com/mikroc/pic32/
 * NOTES:
     - Place DHT22 click board at the mikroBUS socket 1 on the EasyPIC Fusion v7 board.
     - Put EasyTFT in its socket, turn on backlight using the SW11.1
 */

#include <resources.h>

// TFT module connections
char TFT_DataPort at LATE;
sbit TFT_RST at LATD7_bit;
sbit TFT_BLED at LATD2_bit;
sbit TFT_RS at LATD9_bit;
sbit TFT_CS at LATD10_bit;
sbit TFT_RD at LATD5_bit;
sbit TFT_WR at LATD4_bit;
char TFT_DataPort_Direction at TRISE;
sbit TFT_RST_Direction at TRISD7_bit;
sbit TFT_BLED_Direction at TRISD2_bit;
sbit TFT_RS_Direction at TRISD9_bit;
sbit TFT_CS_Direction at TRISD10_bit;
sbit TFT_RD_Direction at TRISD5_bit;
sbit TFT_WR_Direction at TRISD4_bit;
// End TFT module connections

// AM2302 module connections
sbit AM2302_Bus at RC2_bit;
sbit AM2302_Bus_Direction at TRISC2_bit;
// END AM2302 module connections

char AM2302_Data[5] = {0, 0, 0, 0, 0};
unsigned humidity = 0, temperature = 0;

//Get Sensor values

char AM2302_Read(unsigned *humidity, unsigned *temperature) {
  char i = 0, j = 1;
  char timeout = 0;
  char sensor_byte;

  AM2302_Bus_Direction = 0;                           //Set AM2302_Bus as output
  AM2302_Bus = 1;                                     //Set AM2302_Bus HIGH for 100 milliseconds
  Delay_ms(100);                                      //Delay 100ms
  AM2302_Bus = 0;                                     //Host the start signal down time min: 0.8ms, typ: 1ms, max: 20ms
  Delay_ms(2);                                        //Delay 2ms
  AM2302_Bus = 1;                                     //Set AM2302_Bus HIGH
  AM2302_Bus_Direction = 1;                           //Set AM2302_Bus as input

  // Bus master has released time min: 20us, typ: 30us, max: 200us
  timeout = 200;
  while (AM2302_Bus) {
    Delay_us(1);
    if (!timeout--) {
      return 1;
    } //ERROR: Sensor not responding
  }
  
  // AM2302 response signal min: 75us, typ: 80us, max: 85us
  while (!AM2302_Bus) { //response to low time
    Delay_us(1);
  }
  
  while (AM2302_Bus) { //response to high time
    Delay_us(1);
  }
  
  /*
   * time in us:            min  typ  max
   * signal 0 high time:    22   26   30 (bit=0)
   * signal 1 high time:    68   70   75 (bit=1)
   * signal 0,1 down time:  48   50   55
   */
   
  i = 0; //get 5 byte
  for (i = 0; i < 5; i++) {
    j = 1;
    for (j = 1; j <= 8; j++) { //get 8 bits from sensor
      while (!AM2302_Bus) { //signal "0", "1" low time
        Delay_us(1);
      }
      Delay_us(30);
      sensor_byte <<= 1;       //add new lower byte
      if (AM2302_Bus) {     //if sda high after 30us => bit=1 else bit=0
        sensor_byte |= 1;
        delay_us(45);
        while (AM2302_Bus) {
          Delay_us(1);
        }
      }
    }
    AM2302_Data[i] = sensor_byte;
  }
  
  *humidity = (AM2302_Data[0] << 8) + AM2302_Data[1];
  *temperature = (AM2302_Data[2] << 8) + AM2302_Data[3];

  return 0;
}

void Clear_TFT_humidity() {
TFT_Set_Font(&Tahoma15x16_Bold, CL_BLACK, FO_HORIZONTAL);
TFT_Set_Pen(CL_WHITE, 10);
TFT_Set_Brush(1, CL_WHITE, 0, 0, 0, 0);
TFT_Rectangle(170,80,230,95);
TFT_Rectangle(40,160,320,180);
}

void Clear_TFT_temperature() {
TFT_Set_Font(&Tahoma15x16_Bold, CL_BLACK, FO_HORIZONTAL);
TFT_Set_Pen(CL_WHITE, 10);
TFT_Set_Brush(1, CL_WHITE, 0, 0, 0, 0);
TFT_Rectangle(170,100,230,115);
TFT_Rectangle(40,160,320,180);
}

void Clear_TFT_Error() {
TFT_Set_Font(&Tahoma15x16_Bold, CL_RED, FO_HORIZONTAL);
TFT_Set_Pen(CL_WHITE, 10);
TFT_Set_Brush(1, CL_WHITE, 0, 0, 0, 0);
TFT_Rectangle(40,160,320,180);
TFT_Rectangle(170,100,230,115);
TFT_Rectangle(170,80,230,95);
}

// Process and display current value to TFT
void processValue(unsigned humidity, unsigned temperature) {
  char txt[15];
  float temp;

  temp = humidity / 10.0;
  sprintf(txt, "%2.1f", temp);    // Format Relative_humidity_val and store it to txt
  Clear_TFT_humidity();
  TFT_Write_Text(txt,170,80);
  TFT_Write_Char(0x25,200,80);

  temp = temperature / 10.0;
  sprintf(txt, "%2.1f C", temp); // Format UV_val and store it to txt
  Clear_TFT_temperature();
  TFT_Write_Text(txt,170,100);
}

void Display_Init(){
  TFT_Init_ILI9341_8bit(320, 240);
  TFT_BLED = 1;
}

void DrawFrame() {
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_BLACK, 1);
  TFT_Line(20, 220, 300, 220);
  TFT_LIne(20,  46, 300,  46);
  TFT_Set_Font(&Tahoma15x16_Bold, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("DHT22  Click  Board  Demo", 25, 14);
  TFT_Set_Font(&Tahoma15x16_Bold, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("EasyMx PRO v7", 19, 223);
  TFT_Set_Font(&Tahoma15x16_Bold, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.mikroe.com", 200, 223);
  TFT_Set_Font(&Tahoma15x16_Bold, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("Relative Humidity:", 40, 80);
  TFT_Write_Text("Temperature:", 40, 100);
}

void main() {
  char k = 0, t;
  
  Display_Init();
  DrawFrame();

  while (1) {
    if (AM2302_Read(&humidity, &temperature) == 0)                 // Display AM2302_Read sensor values via TFT
      processValue(humidity, temperature);                         // Display AM2302_Read sensor values via TFT
    else {
      Clear_TFT_Error();
      TFT_Write_Text("ERROR: Sensor not responding", 40, 160);
    }
    Delay_ms(2000); //Delay 2000ms
  }
}